#!/bin/sh
#
#
# GOfax: install.sh - simple install script for GOfax 
#
# Author: Lars Scheiter <lars.scheiter@GONICUS.de>
#
# Copyright (C) 2003 GONICUS GmbH
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#    USA
#
# Contact information: GONICUS GmbH
#                      Moehnestrasse 11-17
#                      D-59755 Arnsberg
#                      Germany
#                      tel: ++49 2932 916 0
#                      fax: ++49 2932 916 230
#                      email: info@GONICUS.de
#                      http://www.GONICUS.de
#
# Please dont look at the code, this was written in a complete hurry :)
#

# Default values
PREFIX="/usr/share/gofax"
INSTPRX="/"
BASEFILES="faxrcvd notify"

function repstring(){
	[ ! "$1" -o ! "$2" -o ! "$3" ] && echo "Wrong amount of parameters" && exit 1
	FULLSTRING=`cat "$1" | grep "^$2=\""`
	echo -n . 
	ed $1 2>/dev/null <<EOF
,s%$FULLSTRING%$3%g
wq
EOF
		
}

function rewriteconfig(){
	for y in `find $INSTPRX/$1 ! -type d`; do 
		echo -n ":"
		repstring "$y" "DPREFIX" "DPREFIX=\"$PREFIX\""
		repstring "$y" "HYLAFAX" "HYLAFAX=\"$HYLADIR\""
		repstring "$y" "TMPPREFIX" "TMPPREFIX=\"$TEMPPATH\""
		repstring "$y" "CONFIG" "CONFIG=\"$CONFIGPATH/gofax.config\""
		repstring "$y" "GOFAXCONFIG" "GOFAXCONFIG=\"$CONFIGPATH/gofax.config\""
		repstring "$y" "CONFIGDIR" "CONFIGDIR=\"$CONFIGPATH\""
		repstring "$y" "LOGDIR" "LOGDIR=\"$LOGDIR\""
		repstring "$y" "FUNCTIONDIR" "FUNCTIONDIR=\"$FUNCDIR\""
		repstring "$y" "PLUGINS" "PLUGINS=\"$PLUGINDIR\""
		repstring "$y" "LOCALEDR" "LOCALEDR=\"$LOCALED\""
		repstring "$y" "GOFAXDIR" "GOFAXDIR=\"$PREFIX\""
	done
}

function cpfile(){
	echo -n .
	FILENAME="$1"
	if [ "$3" ]; then
		FILENAME=`echo $1|sed "s/$3\///g"`
	fi
	if [ ! -d $INSTPRX/$2/`dirname $FILENAME` ]; then
		mkdir -m 700 -p $INSTPRX/$2/`dirname $FILENAME`
	fi
	install -m 660 $1 $INSTPRX/$2/$FILENAME
}

function checkdir(){
	[ ! "$1" ] && echo "No path set" && exit 1
	if [ ! -d "$INSTPRX/$1" ]; then
		echo -n "."
		mkdir -p "$INSTPRX/$1"
	else
		echo -n ":"
	fi
	
}

function usage(){
	cat <<EOF
Very simple installer for GOfax
possible options are:
-instprefix	Installation prefix (needed for package building)
-prefix		Global prefix for GOfax
-tempdir	Where should GOfax store its temporary files	
-configdir	Where to install configuration files
-logdir		Where GOfaxs logs are written to
-bindir		Where to install "binary" files
-funcdir	Where to install GOfaxs functions
-docdir		Where to install documentation files
-localedir	Where to install GOfaxs locales 
-contrib	Where to install contrib files
-force		Force installation, no backups will be created! 
install		Start installation

normally you would start this script like this:
./install install -prefix /var/spool/gofax -configdir /etc/gofax
This way all gofax components will will be installed to /var/spool/gofax with 
configuration files written to /etc/gofax.
EOF
exit
}

[ ! "$1" ] && usage

# Check parameters
eval set -- "$@"
for i in "$@" ; do
	case "$1" in
		-prefix)
			if [ -z "$2" ]; then
				echo "No prefix set"
				exit 1
			else
				unset PREFIX
				PREFIX="$2"
			fi
			shift 2
			;;
		-instprefix)
			if [ -z "$2" ]; then
				echo "No prefix set"
				exit 1
			else
				unset INSTPRX 
				INSTPRX="$2"
			fi
			shift 2
			;;
		-tempdir)
			if [ -z "$2" ]; then
				echo "No tepmdir set"
				exit 1
			else
				TEMPPATH="$2"
			fi
			shift 2
			;;
		-logdir)
			if [ -z "$2" ]; then
				echo "No logdir set"
				exit 1
			else
				LOGDIR="$2"
			fi
			shift 2
			;;
		-configdir)
			if [ -z "$2" ]; then
				echo "No config path given"
				exit 1
			else
				CONFIGPATH="$2"
			fi
			shift 2
			;;
		-bindir)
			if [ -z "$2" ]; then
				echo "No binary path given"
				exit 1
			else
				BINPATH="$2"
			fi
			shift 2
			;;
		-funcdir)
			if [ -z "$2" ]; then
				echo "No function path given"
				exit 1
			else
				FUNCDIR="$2"
			fi
			shift 2
			;;
		-plugindir)
			if [ -z "$2" ]; then
				echo "No plugin path given"
				exit 1
			else
				PLUGINDIR="$2"
			fi
			shift 2
			;;
		-docdir)
			if [ -z "$2" ]; then
				echo "No doc path given"
				exit 1
			else
				DOCDIR="$2"
			fi
			shift 2
			;;
		-localedir)
			if [ -z "$2" ]; then
				echo "No locale path given"
				exit 1
			else
				LOCALED="$2"
			fi
			shift 2
			;;
		-contrib)
			if [ -z "$2" ]; then
				echo "No locale path given"
				exit 1
			else
				CONTRIB="$2"
			fi
			shift 2
			;;
		-hyladir)
			if [ -z "$2" ]; then
				echo "No hylafax path given"
				exit 1
			else
				HYLADIR="$2"
			fi
			shift 2
			;;
		-sbinpath)
			if [ -z "$2" ]; then
				echo "No sbin path given"
				exit 1
			else
				SBINPATH="$2"
			fi
			shift 2
			;;
		-force)
			FORCE="YES"
			shift
			;;
		install)
			INSTALL="YES"
			shift
			;;
			
	esac
done
[ ! "$INSTALL" == "YES" ] && usage

# Set default paths if none are provided
[ -z "$CONFIGPATH" ] && CONFIGPATH="$PREFIX/etc"
[ -z "$BINPATH" ] && BINPATH="$PREFIX/bin"
[ -z "$FUNCDIR" ] && FUNCDIR="$PREFIX/functions"
[ -z "$PLUGINDIR" ] && PLUGINDIR="$PREFIX/plugins"
[ -z "$DOCDIR" ] && DOCDIR="$PREFIX/doc"
[ -z "$LOCALED" ] && LOCALED="$PREFIX/locale"
[ -z "$CONTRIB" ] && CONTRIB="$PREFIX/contrib"
[ -z "$LOGDIR" ] && LOGDIR="$PREFIX/logs"
[ -z "$HYLADIR" ] && HYLADIR="/var/spool/fax/bin"
[ -z "$SBINPATH" ] && SBINPATH="/usr/sbin"

QFILEPATH="$INSTPREFIX/$PREFIX/qfiles"

# give warning message and exit if not forced
if [ "$FORCE" == "YES" ]; then
	echo "Forcing Installation of GOfax"
else
cat <<EOF
WARNING: 	This installation program will overwrite some parts of
		your Hylafax Installation. If you proceed you will also 
		invalidate old installations of GOfax, they will NOT be
		upgraded! (Only if installed to the same place)
		If you want to do this start $0 -force {parameters}
EOF
exit 0
fi

# Start here ;)
cat <<EOF
GOfax installer started
Installing GOfax to $PREFIX
Configuration:
	Install prefix		: $INSTPRX
	Configuration files	: $CONFIGPATH
	Binaries		: $BINPATH
	Functions		: $FUNCDIR
	Plugins			: $PLUGINDIR
	Documentation		: $DOCDIR
	Locales			: $LOCALED
	Contrib files		: $CONTRIB
	Logdir			: $LOGDIR
	Hylafax			: $HYLADIR
EOF

# Check for an old installation of GOfax
echo -n "Checking for a previous installation of GOfax "
if [ -d "/var/spool/gofax" -a ! FORCE="YES" ]; then
	echo "Yes"
	echo "Found an old installation of GOfax"
	echo "Creating backup as /var/spool/gofax.backup"
	mv -f /var/spool/gofax /var/spool/gofax.backup
	if [ $? -ne "0" ]; then
		echo "Failed to create backup!"
		echo "Please move the old gofax installation to a new destination"
		echo "and try again"
		exit 1
	else
		echo "Backup created, please recheck configurations after installation"
	fi
else
	echo "No"
fi


# Actually start installing
echo -n "Checking target ."
for i in $INSTPREFIX $INSTPREFIX/$PREFIX $CONFIGPATH $LOGDIR $TEMPPATH $QFILEPATH; do
	checkdir $i
done
echo
echo -n "Installing files ."
echo -n ".gofaxd"
cpfile gofaxd $PREFIX
echo -n ".hylafax"
for i in $BASEFILES; do
	cpfile $i $HYLADIR/bin
done
echo -n ".configuration"
for i in `find configs/ ! -type d|sed "/CVS/d"`; do
	cpfile $i $CONFIGPATH configs
done
echo -n ".bins"
for i in `find bin/ ! -type d|sed "/CVS/d"`; do
	cpfile $i $BINPATH bin
done
echo -n ".functions"
for i in `find functions/ ! -type d|sed "/CVS/d"`; do
	cpfile $i $FUNCDIR functions
done
echo -n ".plugins"
for i in `find plugins/ ! -type d|sed "/CVS/d"`; do
	cpfile $i $PLUGINDIR plugins
done
echo -n ".docs"
for i in `find doc/ ! -type d|sed "/CVS/d"`; do
	cpfile $i $DOCDIR doc
done
echo -n ".locale"
for i in `find locale/ ! -type d|sed "/CVS/d"`; do
	cpfile $i $LOCALED locale
done
echo -n ".contrib"
for i in `find contrib/ ! -type d|sed "/CVS/d"`; do
	cpfile $i $CONTRIB contrib
done
echo -n ".faxsend"
cpfile faxsend $INSTFREFIX/$SBINPATH
echo
echo -n "Configure ."
for i in $CONFIGPATH $BINPATH $FUNCDIR $PLUGINDIR $CONTRIB ; do
	echo -n "#"
	rewriteconfig $i
done
for i in $BASEFILES; do
	echo -n "#"
	rewriteconfig $HYLADIR/bin/$i
done
echo -n "#"
rewriteconfig $PREFIX/gofaxd
rewriteconfig $INSTPREFIX/$SBINPATH/faxsend
echo
cat <<EOF
Hopefully all things are installed correctly. Now go to $CONFIGPATH and customise your GOfax installation.

And always remember:
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
EOF
